#include <runtime/lib.h>
#include <kernel/uos.h>
#include <buf/buf.h>
#include <buf/buf-prio.h>

/*
 *     
 *    .
 */
buf_t *
buf_prio_queue_get (buf_prio_queue_t *q)
{
	buf_queue_t *qq;

	if (q->count <= 0)
		return 0;

	for (qq=q->queue+BUF_PRIO_NQ-1; qq>=q->queue; --qq)
		if (qq->count) {
			--q->count;
			return buf_queue_get (qq);
		}

	/*    -   .
	 *   . */
	q->count = 0;
	return 0;
}

/*
 *       .
 *   -  1,  0.
 */
int
buf_prio_queue_put (buf_prio_queue_t *q, buf_t *p, int prio)
{
	buf_queue_t *qq;

	if (q->count < q->size) {
		/*  . */
		buf_queue_put (&q->queue[prio], p);
		++q->count;
		return 1;
	}

	for (qq=q->queue; qq<=q->queue+prio; ++qq) {
		if (qq->count) {
			/*       
			 *   . */
			buf_free (buf_queue_get (qq));
			buf_queue_put (&q->queue[prio], p);
			return 2;
		}
	}
	/*  . */
	return 0;
}

/*
 *       .
 *        .
 *    .
 */
buf_t *
buf_prio_queue_exchange (buf_prio_queue_t *q, buf_t *p, int prio)
{
	buf_t *r;
	buf_queue_t *qq;

	if (q->count == 0) {
		/*   -   . */
		return p;
	}

	/*        
	 *    . */
	for (qq=q->queue+BUF_PRIO_NQ-1; qq>=q->queue+prio; --qq) {
		if (qq->count) {
			r = buf_queue_get (qq);
			buf_queue_put (&q->queue[prio], p);
			return r;
		}
	}
	/*    -   . */
	return p;
}

void buf_prio_queue_init (buf_prio_queue_t *q, buf_t **buf, int bytes)
{
	int k, bytes_per_queue;

	bytes_per_queue = bytes / BUF_PRIO_NQ;
	q->size = bytes_per_queue / sizeof(buf_t*);
	for (k=0; k<BUF_PRIO_NQ; ++k) {
		buf_queue_init (&q->queue[k], buf, bytes_per_queue);
		buf += q->size;
	}
	q->count = 0;
}
