/*
 * Basic ASN data types.
 */
typedef union _asn_t asn_t;

typedef struct {
	unsigned char	type;
	long		val;
} integer_t;				/* INTEGER */

typedef struct {
	unsigned char	type;
	unsigned long	val;
} uinteger_t;				/* COUNTER */

typedef struct {
	unsigned char	type;
	unsigned short	len;		/* string length */
	unsigned char	str [1];	/* variable length */
} string_t;                             /* OCTET STRING */
#define SIZEOF_STRING(n)	(sizeof(asn_t) + ((n)-1) * sizeof(unsigned char))

typedef struct {
	unsigned char	type;
	unsigned short	len;		/* id length */
	unsigned short	id [1];		/* variable length */
} oid_t;                                /* OBJECT IDENTIFIER */
#define SIZEOF_OID(n)		(sizeof(asn_t) + ((n)-1) * sizeof(unsigned short))

typedef struct {
	unsigned char	type;
	unsigned short	count;		/* number of data elements */
	asn_t *		arr [1];	/* variable length array of elements */
} sequence_t;
#define SIZEOF_SEQUENCE(n)	(sizeof(asn_t) + ((n)-1) * sizeof(asn_t*))

union _asn_t {
	unsigned char	type;
	integer_t	int32;		/* integer */
	uinteger_t	uint32;		/* counter/gauge/timeticks/ipaddress */
	string_t	string;		/* physaddress/opaque */
	oid_t		oid;
	sequence_t	seq;
};

#define ASN_CONSTRUCTED         0x20

/* Universal Class */
#define ASN_INTEGER             0x02
#define ASN_STRING              0x04
#define ASN_NULL                0x05
#define ASN_OID                 0x06
#define ASN_SEQUENCE            0x30

/* Application Class */
#define ASN_IP_ADDRESS          0x40
#define ASN_COUNTER             0x41
#define ASN_GAUGE               0x42
#define ASN_TIME_TICKS          0x43
#define ASN_OPAQUE              0x44

/* Context-Specific Class */
#define ASN_GET_REQUEST		0xa0
#define ASN_GET_NEXT_REQUEST	0xa1
#define ASN_GET_RESPONSE	0xa2
#define ASN_SET_REQUEST		0xa3
#define ASN_TRAP		0xa4
#define ASN_GET_BULK_REQUEST	0xa5
#define ASN_INFORM_REQUEST	0xa6
#define ASN_TRAP_V2		0xa7
#define ASN_REPORT		0xa8

/* SNMPv2 exceptions - RFC 1448. */
#define ASN_NOSUCHOBJECT	0x80
#define ASN_NOSUCHINSTANCE	0x81
#define ASN_ENDOFMIBVIEW	0x82

struct _mem_pool_t;

asn_t *asn_parse (struct _mem_pool_t *pool, unsigned char **input,
	unsigned short *sz);
asn_t *asn_make_oid (struct _mem_pool_t *pool, char *str);
asn_t *asn_make_string (struct _mem_pool_t *pool, char *str);
asn_t *asn_make_stringn (struct _mem_pool_t *pool, char *str,
	unsigned short len);
asn_t *asn_make_int (struct _mem_pool_t *pool, unsigned long val,
	unsigned char type);
asn_t *asn_make_seq (struct _mem_pool_t *pool, unsigned short size,
	unsigned char type);
asn_t *asn_make_oidn (struct _mem_pool_t *pool, unsigned short enterprise,
	const unsigned char *data, unsigned char len, unsigned char cnt, ...);

unsigned char *asn_encode (asn_t*, unsigned char*, unsigned short);
void asn_free (asn_t*);

void asn_print (asn_t*, unsigned short);
void asn_print_type (unsigned char);

asn_t *asn_copy (asn_t*);
