#include <runtime/lib.h>
#include <kernel/uos.h>
#include <timer/timer.h>
#include <net/ip.h>
#include <snmp/asn.h>
#include <snmp/snmp.h>
#include <snmp/snmp-var.h>
#include <snmp/snmp-system.h>

SYSTEM2_VARIABLE_LIST

asn_t *snmp_get_sysDescr (snmp_t *snmp, ...)
{
	return asn_make_string (snmp->pool, snmp->sys_descr);
}

asn_t *snmp_get_sysObjectID (snmp_t *snmp, ...)
{
	return asn_make_oid (snmp->pool, snmp->sys_object_id);
}

asn_t *snmp_get_sysUpTime (snmp_t *snmp, ...)
{
	unsigned long santisec = 0;

	if (snmp->ip->timer)
		santisec = timer_milliseconds (snmp->ip->timer) / 10;
	return asn_make_int (snmp->pool, santisec, ASN_TIME_TICKS);
}

asn_t *snmp_get_sysContact (snmp_t *snmp, ...)
{
	return asn_make_string (snmp->pool, snmp->sys_contact);
}

unsigned char snmp_set_sysContact (snmp_t *snmp, asn_t *val, ...)
{
	unsigned char len;

	if (val->type != ASN_STRING || val->string.len <= 0)
		return SNMP_BAD_VALUE;

	if (val->string.len < sizeof (snmp->sys_contact))
		len = val->string.len;
	else
		len = sizeof (snmp->sys_contact) - 1;

	strncpy (snmp->sys_contact, (char*) val->string.str, len);
	snmp->sys_contact [len] = 0;

	if (snmp_change_sysContact && ! snmp_change_sysContact (snmp))
		return SNMP_GEN_ERR;
	return SNMP_NO_ERROR;
}

asn_t *snmp_get_sysName (snmp_t *snmp, ...)
{
	return asn_make_string (snmp->pool, snmp->sys_name);
}

unsigned char snmp_set_sysName (snmp_t *snmp, asn_t *val, ...)
{
	unsigned char len;

	if (val->type != ASN_STRING || val->string.len <= 0)
		return SNMP_BAD_VALUE;

	if (val->string.len < sizeof (snmp->sys_name))
		len = val->string.len;
	else
		len = sizeof (snmp->sys_name) - 1;

	strncpy (snmp->sys_name, (char*) val->string.str, len);
	snmp->sys_name [len] = 0;

	if (snmp_change_sysName && ! snmp_change_sysName (snmp))
		return SNMP_GEN_ERR;
	return SNMP_NO_ERROR;
}

asn_t *snmp_get_sysLocation (snmp_t *snmp, ...)
{
	return asn_make_string (snmp->pool, snmp->sys_location);
}

unsigned char snmp_set_sysLocation (snmp_t *snmp, asn_t *val, ...)
{
	unsigned char len;

	if (val->type != ASN_STRING || val->string.len <= 0)
		return SNMP_BAD_VALUE;

	if (val->string.len < sizeof (snmp->sys_location))
		len = val->string.len;
	else
		len = sizeof (snmp->sys_location) - 1;

	strncpy (snmp->sys_location, (char*) val->string.str, len);
	snmp->sys_location [len] = 0;

	if (snmp_change_sysLocation && ! snmp_change_sysLocation (snmp))
		return SNMP_GEN_ERR;
	return SNMP_NO_ERROR;
}

asn_t *snmp_get_sysServices (snmp_t *snmp, ...)
{
	return asn_make_int (snmp->pool, snmp->sys_services, ASN_INTEGER);
}

asn_t *snmp_get_sysORLastChange (snmp_t *snmp, ...)
{
	return asn_make_int (snmp->pool, 0, ASN_TIME_TICKS);
}

asn_t *
snmp_get_sysORID (snmp_t *snmp, unsigned short id, ...)
{
	if (id != 1)
		return 0;
	return asn_make_oid (snmp->pool, snmp->sys_resource_id);
}

asn_t *
snmp_next_sysORID (snmp_t *snmp, unsigned char nextflag, unsigned short *id, ...)
{
	if (nextflag)
		++*id;
	else
		*id = 1;
	if (*id != 1)
		return 0;
	return asn_make_oid (snmp->pool, snmp->sys_resource_id);
}

asn_t *
snmp_get_sysORDescr (snmp_t *snmp, unsigned short id, ...)
{
	if (id != 1)
		return 0;
#if __AVR__
	{
	char buf [80], *s, *d;

	/* Copy from flash. */
	s = snmp->sys_resource_descr;
	d = buf;
	while ((*d++ = FETCH(s)) != '\0') {
		if (d == &buf [sizeof(buf)-1]) {
			*d = '\0';
			break;
		}
		++s;
	}
	return asn_make_string (snmp->pool, buf);
	}
#else
	return asn_make_string (snmp->pool, snmp->sys_resource_descr);
#endif
}

asn_t *
snmp_next_sysORDescr (snmp_t *snmp, unsigned char nextflag, unsigned short *id, ...)
{
	if (nextflag)
		++*id;
	else
		*id = 1;
	return snmp_get_sysORDescr (snmp, *id);
}

asn_t *
snmp_get_sysORUpTime (snmp_t *snmp, unsigned short id, ...)
{
	if (id != 1)
		return 0;
	return asn_make_int (snmp->pool, 0, ASN_TIME_TICKS);
}

asn_t *
snmp_next_sysORUpTime (snmp_t *snmp, unsigned char nextflag, unsigned short *id, ...)
{
	if (nextflag)
		++*id;
	else
		*id = 1;
	if (*id != 1)
		return 0;
	return asn_make_int (snmp->pool, 0, ASN_TIME_TICKS);
}
